#ifndef __CUBE_MAP__
#define __CUBE_MAP__

#include <GlewOpenGL.h>
#include "TextureData.h"
#include "Image2D.h"
#include "ColorTypes.h"
#include "Vec3.h"
#include <QString>
#include <QStringList>

namespace apig {

    /** This class represents data for a cube-map. */
    template<class Color>
    class CubeMap : public TextureData {
        public:
            CubeMap() : loaded(false) {}
            CubeMap(QString dirName);
        
            void initialize(QString dirName);
            void destroy();
            
            // emulation de l'echantillonnage de cube-maps OpenGL :
            Color sample(float rx, float ry, float rz) { return sample(Vec3(rx,ry,rz)); }
            Color sample(Vec3 r);
            
            virtual GLenum textureMode() const { return GL_TEXTURE_CUBE_MAP; }
            virtual GLint defaultTexFormat() const { return face[0][0].defaultTexFormat(); }
            virtual void loadToGPU(GLint texFormat) const;
            
            static QStringList namesList(QString dirName, QString first = QString::null);     // liste des cubemaps dans le repertoire <dirName>, avec eventuellement <first> en premier
            
        private:
            static GLenum targetName(int i, int j);       // i:{x=0,y=1,z=2}, j:{neg=0,pos=1}
            static QString imageFileName(int i, int j);   // i:{x=0,y=1,z=2}, j:{neg=0,pos=1}
            
        private:
            bool loaded;
            Image2D<Color> face[3][2];
            
            //static const QString defaultDir;
            static const QString axisNames[3];      // x, y, z
            static const QString signNames[2];      // neg, pos
            
        public:
            static const GLenum targetNames[6];
        };
    }

#include "CubeMap_impl.h"

#endif

